@@ -112,7 +112,8 @@ class InterpreterClass(_instance_base.SimpleValue, class_mixin.Class):
     # instances created by analyze.py for the purpose of analyzing this class,
     # a subset of 'instances'. Filled through register_canonical_instance.
     self.canonical_instances = set()
-    self.slots = self._convert_slots(members.get("__slots__"))
+    self.slots = self._convert_str_tuple(members, "__slots__")
+    self.match_args = self._convert_str_tuple(members, "__match_args__") or ()
     self.is_dynamic = self.compute_is_dynamic()
     log.info("Created class: %r", self)
     self.type_param_check()
@@ -227,14 +228,15 @@ class InterpreterClass(_instance_base.SimpleValue, class_mixin.Class):
     else:
       return name
 
-  def _convert_slots(self, slots_var):
-    """Convert __slots__ from a Variable to a tuple."""
-    if slots_var is None:
+  def _convert_str_tuple(self, members, field_name):
+    """Convert __slots__ and similar fields from a Variable to a tuple."""
+    field_var = members.get(field_name)
+    if field_var is None:
       return None
-    if len(slots_var.bindings) != 1:
+    if len(field_var.bindings) != 1:
       # Ambiguous slots
       return None  # Treat "unknown __slots__" and "no __slots__" the same.
-    val = slots_var.data[0]
+    val = field_var.data[0]
     if isinstance(val, mixin.PythonConstant):
       if isinstance(val.pyval, (list, tuple)):
         entries = val.pyval
@@ -250,7 +252,7 @@ class InterpreterClass(_instance_base.SimpleValue, class_mixin.Class):
     for s in names:
       if not isinstance(s, str):
         self.ctx.errorlog.bad_slots(self.ctx.vm.frames,
-                                    f"Invalid __slot__ entry: {str(s)!r}")
+                                    f"Invalid {field_name} entry: {str(s)!r}")
         return None
     return tuple(self._mangle(s) for s in names)
 
@@ -288,6 +290,13 @@ class InterpreterClass(_instance_base.SimpleValue, class_mixin.Class):
     annotations_dict = abstract_utils.get_annotations_dict(self.members)
     return annotations_dict and name in annotations_dict.annotated_locals
 
+  def has_protocol_base(self):
+    for base_var in self._bases:
+      for base in base_var.data:
+        if isinstance(base, PyTDClass) and base.full_name == "typing.Protocol":
+          return True
+    return False
+
 
 class PyTDClass(
     _instance_base.SimpleValue, class_mixin.Class, mixin.LazyMembers):
@@ -509,6 +518,12 @@ class PyTDClass(
                   self.ctx.root_node, container=instance)
         return self._convert_member(name, c, subst)
 
+  def has_protocol_base(self):
+    for base in self.pytd_cls.bases:
+      if base.name == "typing.Protocol":
+        return True
+    return False
+
 
 class FunctionPyTDClass(PyTDClass):
   """PyTDClass(Callable) subclass to support annotating higher-order functions.
@@ -750,6 +765,9 @@ class ParameterizedClass(  # pytype: disable=signature-mismatch
       typ = self.__class__
     return typ(self.base_cls, inner_types, self.ctx, self.template)
 
+  def has_protocol_base(self):
+    return self.base_cls.has_protocol_base()
+
 
 class CallableClass(ParameterizedClass, mixin.HasSlots):  # pytype: disable=signature-mismatch
   """A Callable with a list of argument types.
@@ -836,8 +854,11 @@ class LiteralClass(ParameterizedClass):
 
   def __eq__(self, other):
     if isinstance(other, LiteralClass):
-      if self.value and other.value:
+      if (isinstance(self.value, mixin.PythonConstant) and
+          isinstance(other.value, mixin.PythonConstant)):
         return self.value.pyval == other.value.pyval
+      else:
+        return self.value == other.value
     return super().__eq__(other)
 
   def __hash__(self):
