@@ -226,8 +226,6 @@ FEATURE_FLAGS = [
           "Enable parameter count checks for overriding methods."),
     _flag("--overriding-parameter-name-checks", False,
           "Enable parameter name checks for overriding methods."),
-    _flag("--overriding-parameter-type-checks", True,
-          "Enable parameter type checks for overriding methods."),
     _flag("--overriding-return-type-checks", False,
           "Enable return type checks for overriding methods."),
     _flag("--enable-cached-property", False,
